﻿using System;
using System.Diagnostics;
using System.Threading;

namespace Threads
{
    class Program
    {

        static void OurThreadWithoutParameters()
        {
            Console.WriteLine("Thread without parameters");
        }
        static void OurThread()
        {
            Console.WriteLine($"Overloaded method Thread without parameeter");
        }
        static void OurThread(object o)
        {
            string message = o as string; 
            Console.WriteLine($"Overloaded method Thread with parameeter got message: {message}");
        }
        static void OurThreadWithParameter(object o)
        {
            if (o == null)
            {
                Console.WriteLine("I got null as a parameter"); 
                return;
            }
            Console.WriteLine("Thread with parameter got message: " + (string)o);
        }

        static void OurThreadWithTypedParameter(string message)
        {
            Console.WriteLine("Thread with typed parameter got message: " + message);
        }


        static void Main(string[] args)
        {
            Console.WriteLine("-------------- Threads without parameter ---------------");
            //Creating threads without parameters
            Thread thread1 = new Thread(new ThreadStart(OurThreadWithoutParameters));
            Thread thread2 = new Thread(OurThreadWithoutParameters); //compilator adds automaticly new(ThreadStart 
            Thread thread3 = new Thread(new ThreadStart(OurThread));
            //Thread thread3 = new Thread(OurThread); //When we have overloaded methods (without and with parameters, the compiler does not know whether to use ThreadStart or ParameterizedThreadStart
            thread1.Start();
            thread2.Start();
            thread3.Start();
            //thread1.Start("This is a wrong parameter");//Invalid operation exception
            
            Console.ReadLine();
            Console.WriteLine("-------------- Threads with parameter ---------------");
            Thread thread4 = new Thread(new ParameterizedThreadStart(OurThreadWithParameter));
            Thread thread5 = new Thread(OurThreadWithParameter);
            Thread thread6 = new Thread(new ParameterizedThreadStart(OurThread));

            thread4.Start(); //Possible such a start, but there will be no parameters o = null
            thread5.Start("Message for thread 5");
            thread6.Start("Message for thread 6");

            Console.ReadLine();
            Console.WriteLine("-------------- Threads anonymous start and changing message ---------------");
            string changingMessage;
            changingMessage = "Message befor thread created";
            Thread thread7 = new Thread(delegate () { OurThreadWithTypedParameter(changingMessage); }); //using an anonymous method, we do not have to specify the object parameter, but we can specify a specific type, e.g. string
            changingMessage = "Message after thread chreated";
            Thread thread8 = new Thread(delegate() 	
            {
                Console.WriteLine(changingMessage);
            });
            thread7.Start();
            thread8.Start();

            Console.ReadLine();
            Console.WriteLine("-------------- Thread created with lambda notation ---------------");

            Thread thread9 = new Thread((parameter) =>
            {  //thread code
                Console.WriteLine("Lambda thread show message: " + (parameter as string));
            });
            thread9.Start("Some message for lambda created thread");

            Thread thread10 = new Thread((parameter) =>
            {  //thread code
            OurThreadWithTypedParameter(parameter as string);
            });
            thread10.Start("Some message for lambda created thread with method");

            Console.ReadLine();
            Console.WriteLine("-------------- Thread naming ---------------");
            Thread thread11 = new Thread(() =>
            Console.WriteLine($"Hello. I'm {Thread.CurrentThread.Name} thread.")
            );
            thread11.Name = "Eleven";
            thread11.Start();


            Console.ReadLine();
            Console.WriteLine("-------------- Background Thread ---------------");
            Thread parentThread = new Thread(() =>
            {
                Console.WriteLine($"Thread {Thread.CurrentThread.Name} started");
                Thread childThread1 = new Thread(() =>
                {
                    Console.WriteLine($"Thread {Thread.CurrentThread.Name} started");
                    Thread.Sleep(4000);
                    Console.WriteLine($"Thread {Thread.CurrentThread.Name} finished");
                })
                { Name = "ChildThread1"}; //backgroud = false as default
                //{ Name = "ChildThread1",IsBackground = true };
                Thread childThread2 = new Thread(() =>
                {
                    Console.WriteLine($"Thread {Thread.CurrentThread.Name} started");
                    Thread.Sleep(5000);
                    Console.WriteLine($"Thread {Thread.CurrentThread.Name} finished");
                })
                //{ Name = "ChildThread2"};//backgroud = false as default
                { Name = "ChildThread2",IsBackground=true };

                childThread1.Start();
                childThread2.Start();
                Console.WriteLine($"Thread {Thread.CurrentThread.Name} finished");
                
            })
            { Name = "Parent" };
            parentThread.Start();
             
        }
    }
}
